/* ----------------------------------------------------------------------- */
/*                                                                         */
/* [klt.h]                      Klt                                   */
/*                                                                            */
/* -------------------------------------------------------------------------- */
/*                                                                            */
/* Copyright (c) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>            */
/*                                                                            */
/* This file is part of Klt, an X terminal.                               */
/*                                                                            */
/* The material contained in here more or less directly orginates from        */
/* kvt, which is copyright (c) 1996 by Matthias Ettrich <ettrich@kde.org>     */
/*                                                                            */
/* -------------------------------------------------------------------------- */

#ifndef KONSOLE_H
#define KONSOLE_H


#include <tdemainwindow.h>
#include <kdialogbase.h>
#include <ksimpleconfig.h>
#include <tdeaction.h>
#include <tdepopupmenu.h>
#include <keditcl.h>

#include <twinmodule.h>

#include <tqstrlist.h>
#include <tqintdict.h>
#include <tqptrdict.h>

#include "TEPty.h"
#include "TEWidget.h"
#include "TEmuVt102.h"
#include "session.h"
#include "schema.h"
#include "klt_child.h"

#include "kltiface.h"

#undef PACKAGE
#undef VERSION
#define PACKAGE "klt"
#define VERSION "1.2"

class KRootPixmap;
class TQCheckBox;
class KltFind;

// Defined in main.C
const char *klt_shell(TQStrList &args);

class Klt : public TDEMainWindow, virtual public KltIface
{
    Q_OBJECT

    friend class KltSessionManaged;
public:

  Klt(int isrestored );
  Klt(const TQString &pgm, const TQStrList &args,int isrestored );
  ~Klt();
  void setColLin(int columns, int lines);
  void setAutoClose(bool on);
  void initSessionFont(int fontNo);
  void initSessionTitle(const TQString &_title);
  void initSessionKeyTab(const TQString &keyTab);
	void initFullScreen();
  void initMasterMode(bool on);
  void newSession(const TQString &program, const TQStrList &args, const TQString &term, const TQString &icon, const TQString &cwd);
  void setSchema(const TQString & path);
  
  void enableFullScripting(bool b);

  void run();
  void setDefaultSession(const TQString &filename);
  void showTipOnStart();


  // Additional functions for DCOP
  int sessionCount() { return sessions.count(); }

  TQString currentSession();
  TQString newSession(const TQString &type);
  TQString sessionId(const int position);

  void activateSession(const TQString& sessionId);
  void feedAllSessions(const TQString &text);
  void sendAllSessions(const TQString &text);

  KURL baseURL() const;

  virtual bool processDynamic(const TQCString &fun, const TQByteArray &data, TQCString& replyType, TQByteArray &replyData);
  virtual QCStringList functionsDynamic();

public slots:
  void activateSession(const int position);

  void makeGUI();
  TQString newSession();

protected:

 bool queryClose();
 bool resizeEvent();
 bool queryExit();
 void saveProperties(TDEConfig* config);
 void readProperties(TDEConfig* config);

 

private slots:
  void currentDesktopChanged(int desk);
  void slotBackgroundChanged(int desk);
  void configureRequest(TEWidget*,int,int,int);
  void activateSession();
  void activateSession(TESession*);
  void closeCurrentSession();
  void doneChild(KltChild*, TESession*);
  void doneSession(TESession*);
  void slotCouldNotClose();
  void schema_menu_activated(int item);
  void pixmap_menu_activated(int item);
  void keytab_menu_activated(int item);
  void schema_menu_check();
  void attachSession(TESession*);
  void detachSession();
  void newSession(int kind);
  void updateSchemaMenu();
  void updateKeytabMenu();

  void changeColumns(int);
  void changeLines(int);
  void notifySessionState(TESession* session,int state);
  void notifySize(int,int);
  void updateTitle();
  void activateMenu();
  void activateStatusBar(const TQString &msg);
  void clearStatusBar();
  void setSchema(int n);
  void sendSignal(int n);
  void slotClearTerminal();
  void slotResetClearTerminal();
  void slotToggleMenubar();
  void slotToggleMasterMode();
  void slotSelectBell();
  void slotSelectAutoresize();
  void slotSelectSize();
  void slotSelectFont();
  void loadScreenSessions();

  void slotSaveSettings();
  void slotSaveSessionsProfile();
  void slotConfigureKeys();
  void slotConfigure();
  void reparseConfiguration();

  void clearAllListenToKeyPress();
  void restoreAllListenToKeyPress();
  void newSession( const TQString&, const TQString& );

  void fontNotFound();
  void showTip();

  void listSessions();
  void switchToSession1() { activateSession(1); }
  void switchToSession2() { activateSession(2); }
  void switchToSession3() { activateSession(3); }
  void switchToSession4() { activateSession(4); }
  void switchToSession5() { activateSession(5); }
  void switchToSession6() { activateSession(6); }
  void switchToSession7() { activateSession(7); }
  void switchToSession8() { activateSession(8); }
  void switchToSession9() { activateSession(9); }
  void switchToSession10() { activateSession(10); }
  void switchToSession11() { activateSession(11); }
  void switchToSession12() { activateSession(12); }

  void biggerFont();
  void smallerFont();

private:
  KSimpleConfig *defaultSession();
  TQString newSession(KSimpleConfig *co, TQString pgm = TQString::null, const TQStrList &args = TQStrList(), const TQString &_term = TQString::null, const TQString &_icon = TQString::null, const TQString &_title = TQString::null, const TQString &_cwd = TQString::null,const TQString &_font = TQString::null );
  void readProperties(TDEConfig *config, const TQString &schema, bool globalConfigOnly);
  void applySettingsToGUI();
  void makeBasicGUI();
  void runSession(TESession* s);
  void addSession(TESession* s);
  void setColorPixmaps();

  void setSchema(ColorSchema* s);
  void setFont(int fontno);

  void buildSessionMenus();
  void addSessionCommand(const TQString & path);
  void loadSessionCommands();
  void addScreenSession(const TQString & path, const TQString & socket);
  void resetScreenSessions();

  TQPtrDict<TESession> action2session;
  TQPtrDict<TDERadioAction> session2action;
  TQPtrList<TESession> sessions;
  TQPtrList<KltChild> detached;
  TQIntDict<KSimpleConfig> no2command;
  TQIntDict<KTempFile> no2tempFile;
  TQIntDict<TQString> no2filename;
  KSimpleConfig* m_defaultSession;
  TQString m_defaultSessionFilename;

  TEWidget*      te;
  TESession*     se;
  TESession*     se_previous;
  TESession*     m_initialSession;
  ColorSchemaList* colors;

  KRootPixmap*   rootxpm;
  KWinModule*    kWinModule;

  KMenuBar*   menubar;
  KStatusBar* statusbar;

  TDEPopupMenu* m_session;
  TDEPopupMenu* m_edit;
  TDEPopupMenu* m_view;
  TDEPopupMenu* m_options;
  TDEPopupMenu* m_schema;
  TDEPopupMenu* m_keytab;
  TDEPopupMenu* m_signals;
  TDEPopupMenu* m_help;
  TDEPopupMenu* m_rightButton;

  TDEToggleAction *masterMode;
  TDEToggleAction *showMenubar;

  TDESelectAction *selectSize;
  TDESelectAction *selectFont;
  TDESelectAction *selectBell;
  TDESelectAction *selectAutoresize;

  TDEAction       *m_detachSession;
  TDEAction       *m_moveSessionLeft;
  TDEAction       *m_moveSessionRight;

  TDEAction       *m_copyClipboard;
  TDEAction       *m_pasteClipboard;
  TDEAction       *m_clearTerminal;
  TDEAction       *m_resetClearTerminal;
  TDEAction       *m_clearAllSessionHistories;
  TDEAction       *m_saveProfile;
  TDEAction       *m_closeSession;
  TDEAction       *m_quit;

  TDEActionCollection *m_shortcuts;

  KltFind* m_finddialog;
  bool         m_find_first;
  bool         m_find_found;
  TQString      m_find_pattern;

  int cmd_serial;
  int cmd_first_screen;
  int         n_keytab;
  int         n_defaultKeytab;
  int         n_font;
  int         n_defaultFont; // font as set in config to use as default for new sessions
  int         n_scroll;
  int         n_bell;
  int         n_autoresize;
  int         n_render;
  int         curr_schema; // current schema no
  int         wallpaperSource;
  int         sessionIdCounter;
  int         monitorSilenceSeconds;

  TQString     s_schema;
  TQString     s_tdeconfigSchema;
  TQString     s_word_seps;			// characters that are considered part of a word
  TQString     pmPath; // pixmap path
  TQString     dropText;
  TQString     fontNotFound_par;
  TQFont       defaultFont;
  TQSize       defaultSize;

  TQRect       _saveGeometry;

  TQTimer      m_closeTimeout;

  bool        b_framevis:1;
  bool        m_menuCreated:1;
  bool        skip_exit_query:1;
  bool        b_warnQuit:1;
  bool        isRestored:1;
  bool        b_allowResize:1;
  bool        b_addToUtmp:1;
  bool        b_xonXoff:1;

  bool        b_histEnabled:1;
  bool        b_fullScripting:1;
  unsigned int m_histSize;
};

class TQSpinBox;

class SizeDialog : public KDialogBase
{
    Q_OBJECT
public:
  SizeDialog(unsigned int const columns,
             unsigned int const lines,
             TQWidget *parent);

public slots:
  void slotDefault();

  unsigned int columns() const;
  unsigned int lines() const;

protected:
  TQSpinBox*  m_columns;
  TQSpinBox*  m_lines;
};

class KltFind : public KEdFind
{
    Q_OBJECT
public:
  KltFind( TQWidget *parent = 0, const char *name=0, bool modal=true );
  bool reg_exp() const;

private slots:
  void slotEditRegExp();

private:
  TQCheckBox*    m_asRegExp;
  TQDialog*      m_editorDialog;
  TQPushButton*  m_editRegExp;
};

#endif
