/* -------------------------------------------------------------------------- */
/*                                                                            */
/* [session.h]              Testbed for TE framework                          */
/*                                                                            */
/* -------------------------------------------------------------------------- */
/*                                                                            */
/* Copyright (c) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>            */
/*                                                                            */
/* This file is part of Klt, an X terminal.                               */
/*                                                                            */
/* -------------------------------------------------------------------------- */

#ifndef SESSION_H
#define SESSION_H

#include <tdeapplication.h>
#include <tdemainwindow.h>
#include <tqstrlist.h>

#include <TEPty.h>
#include <TEWidget.h>
#include <TEmuVt102.h>

#include "sessioniface.h"

class TESession : public TQObject, virtual public SessionIface
{ Q_OBJECT

public:

  TESession(TEWidget* w,
            const TQString &pgm, const TQStrList & _args,
	    const TQString &term, const TQString &sessionId="session-1",
	    const TQString &initial_cwd = TQString::null);
  void changeWidget(TEWidget* w);
  ~TESession();

  void        setConnect(bool r);  // calls setListenToKeyPress(r)
  void        setListenToKeyPress(bool l);
  TEmulation* getEmulation();      // to control emulation
  bool        isSecure();
  bool        isMasterMode();
  int schemaNo();
  int fontNo();
  const TQString& Term();
  const TQString& SessionId();
  const TQString& Title();
  const TQString& IconName();
  const TQString& IconText();
  TQString fullTitle() const;
  int keymapNo();
  TQString keymap();
  TQStrList getArgs();
  TQString getPgm();
  TQString getCwd();
  TQString getInitial_cwd() { return initial_cwd; }
  void setInitial_cwd(const TQString& _cwd) { initial_cwd=_cwd; }

  void setHistory(const HistoryType&);
  const HistoryType& history();

  void setMasterMode(bool);
  void setSchemaNo(int sn);
  void setKeymapNo(int kn);
  void setKeymap(const TQString& _id);
  void setFontNo(int fn);
  void setTitle(const TQString& _title);
  void setIconName(const TQString& _iconName);
  void setIconText(const TQString& _iconText);
  void setAddToUtmp(bool);
  void setXonXoff(bool);
  bool testAndSetStateIconName (const TQString& newname);
  bool sendSignal(int signal);

  void setAutoClose(bool b) { autoClose = b; }

  // Additional functions for DCOP
  bool closeSession();
  void feedSession(const TQString &text);
  void sendSession(const TQString &text);

  virtual bool processDynamic(const TQCString &fun, const TQByteArray &data, TQCString& replyType, TQByteArray &replyData);
  virtual QCStringList functionsDynamic();
  void enableFullScripting(bool b) { fullScripting = b; }

public slots:

  void run();
  void done();
  void terminate();
  void setUserTitle( int, const TQString &caption );
  void ptyError();

signals:

  void done(TESession*);
  void updateTitle();
  void notifySessionState(TESession* session, int state);

  void clearAllListenToKeyPress();
  void restoreAllListenToKeyPress();

  void openURLRequest(const TQString &cwd);

private slots:
  void monitorTimerDone();
  void notifySessionState(int state);

private:

  TEPty*         sh;
  TEWidget*      te;
  TEmulation*    em;

  bool           masterMode;
  bool           autoClose;
  TQTimer*        monitorTimer;

  //FIXME: using the indices here
  // is propably very bad. We should
  // use a persistent reference instead.
  int            schema_no;
  int            font_no;
  int            silence_seconds;

  TQString        title;
  TQString        userTitle;
  TQString        iconName;
  TQString        iconText; // as set by: echo -en '\033]1;IconText\007
  bool           add_to_utmp;
  bool           xon_xoff;
  bool           fullScripting;

  TQString	 stateIconName;

  TQString        pgm;
  TQStrList       args;

  TQString        term;
  TQString        sessionId;

  TQString        cwd;
  TQString        initial_cwd;
};

#endif
